@@ -4,6 +4,7 @@
 from __future__ import print_function
 
 import datetime
+import enum
 import json
 import os
 import re
@@ -32,6 +33,17 @@ _DISPLAY_RE = re.compile(
     r'.*DisplayViewport{valid=true, .*orientation=(?P<orientation>\d+), .*deviceWidth=(?P<width>\d+), deviceHeight=(?P<height>\d+).*'
 )
 
+class Network(str, enum.Enum):
+    TCP = "tcp"
+    UNIX = "unix"
+
+    DEV = "dev"
+    LOCAL = "local"
+    LOCAL_RESERVED = "localreserved"
+    LOCAL_FILESYSTEM = "localfilesystem"
+    LOCAL_ABSTRACT = "localabstract" # same as UNIX
+
+
 DeviceEvent = namedtuple('DeviceEvent', ['present', 'serial', 'status'])
 ForwardItem = namedtuple("ForwardItem", ["serial", "local", "remote"])
 ReverseItem = namedtuple("ReverseItem", ["remote", "local"])
@@ -207,6 +219,20 @@ class AdbClient(object):
             c.check_okay()
             return c.read_string_block()
 
+    def disconnect(self, addr: str) -> str:
+        """ adb disconnect $addr
+        Returns:
+            content adb server returns
+
+        Example returns:
+            - "disconnected 192.168.190.101:5555"
+        """
+
+        with self._connect() as c:
+            c.send_command("host:disconnect:" + addr)
+            c.check_okay()
+            return c.read_string_block()
+
     def shell(self,
               serial: str,
               command: Union[str, list, tuple],
@@ -230,6 +256,8 @@ class AdbClient(object):
             command = subprocess.list2cmdline(command)
         assert isinstance(command, six.string_types)
         c = self._connect()
+        # when no response in timeout, socket.timeout will raise
+        c.conn.settimeout(timeout)
         try:
             c.send_command("host:transport:" + serial)
             c.check_okay()
@@ -237,9 +265,6 @@ class AdbClient(object):
             c.check_okay()
             if stream:
                 return c
-
-            # when no response in timeout, socket.timeout will raise
-            c.conn.settimeout(timeout)
             try:
                 return c.read_until_close()
             except socket.timeout:
@@ -517,6 +542,34 @@ class AdbDevice(ShellMixin):
     def push(self, local: str, remote: str):
         self.adb_output("push", local, remote)
 
+    def create_connection(self, network: Network, address: Union[int, str]):
+        """
+        Used to connect a socket (unix of tcp) on the device
+
+        Returns:
+            socket object
+
+        Raises:
+            AssertionError, ValueError
+        """
+        c = self._client._connect()
+        c.send_command("host:transport:" + self._serial)
+        c.check_okay()
+        if network == Network.TCP:
+            assert isinstance(address, int)
+            c.send_command("tcp:" + str(address))
+            c.check_okay()
+        elif network in [Network.UNIX, Network.LOCAL_ABSTRACT]:
+            assert isinstance(address, str)
+            c.send_command("localabstract:" + address)
+            c.check_okay()
+        elif network in [Network.LOCAL_FILESYSTEM, Network.LOCAL, Network.DEV, Network.LOCAL_RESERVED]:
+            c.send_command(network + ":" + address)
+            c.check_okay()
+        else:
+            raise ValueError("Unsupported network type", network)
+        return c.conn
+    
 
 class Sync():
     def __init__(self, adbclient: AdbClient, serial: str):
@@ -543,8 +596,9 @@ class Sync():
         with self._prepare_sync(path, "STAT") as c:
             assert "STAT" == c.read_string(4)
             mode, size, mtime = struct.unpack("<III", c.conn.recv(12))
-            return FileInfo(mode, size, datetime.datetime.fromtimestamp(mtime),
-                            path)
+            # when mtime is 0, windows will error
+            mdtime = datetime.datetime.fromtimestamp(mtime) if mtime else None
+            return FileInfo(mode, size, mdtime, path)
 
     def iter_directory(self, path: str):
         with self._prepare_sync(path, "LIST") as c:
