@@ -115,6 +115,7 @@ def main():
     parser.add_argument("--list-packages",
                         action="store_true",
                         help="list packages installed")
+    parser.add_argument("--current", action="store_true", help="show current package info")
     parser.add_argument("-p",
                         "--package",
                         help="show package info in json format")
@@ -299,10 +300,15 @@ def main():
     elif args.browser:
         d.open_browser(args.browser)
 
+    elif args.current:
+        package_name = d.current_app()['package']
+        info = d.package_info(package_name)
+        print(json.dumps(info, indent=4, default=str))
     elif args.package:
         info = d.package_info(args.package)
         print(json.dumps(info, indent=4, default=str))
 
 
+
 if __name__ == "__main__":
     main()
